/*
 * Programming in Java, Jacobs University, 2015
 *
 * This file demonstrates how to implement a simple chat server. The
 * ServeChatClientOutput class implements a thread that essentially
 * waits for input from the dispatcher and then copies it over to the
 * client.
 */

import java.net.Socket;
import java.io.Reader;
import java.io.Writer;
import java.io.IOException;
import java.io.OutputStreamWriter;

import java.util.concurrent.BlockingQueue;

public class ServeChatClientOutput
    implements Runnable
{
    private Socket socket;
    private OutputStreamWriter out;
    private ChatDispatcher dispatcher;
    private BlockingQueue<String> queue;

    public ServeChatClientOutput(Socket s, ChatDispatcher d,
				 BlockingQueue<String> q)
	throws IOException
    {
	socket = s;
	dispatcher = d;
	queue = q;
	out = new OutputStreamWriter(socket.getOutputStream());
    }
    
    public void run()
    {
	try {
	    while (! Thread.currentThread().isInterrupted()) {
		try {
		    String s = queue.take();
		    out.write(s, 0, s.length());
		    out.write("\n", 0, 1);
		    out.flush();
		} catch (IOException e) {
		    Thread.currentThread().interrupt();
		} catch (InterruptedException e) {
		    // reassert the interrupt to maintain the interrupt state
		    Thread.currentThread().interrupt();
		}
	    }
	} finally {
	    try {
		socket.close();
	    } catch (IOException ignored) { }
	    dispatcher.removeClient(socket);
	}
    }
}
